﻿//+------------------------------------------------------------------+
//|                                              Blue Renko Bars.mq5 |
//+------------------------------------------------------------------+
#property copyright     "© SharmuttaDJ, 2018-2024"
#property link          "https://www.mql5.com/ru/users/sharmuttadj/"
#property version       "2.0"
#property description   "Simple Renko Indicator for MetaTrader 5\n"
#property description   "With improvements and bug fixes by Marketeer\n"

// indicator settings
#property indicator_separate_window
#property indicator_buffers 4
#property indicator_plots   1

// plot Renko
#property indicator_label1  "Open;High;Low;Close"
#property indicator_type1   DRAW_CANDLES
#property indicator_color1  clrDodgerBlue,clrBlack,clrDeepSkyBlue
#property indicator_style1  STYLE_SOLID
#property indicator_width1  1

// indicator inputs
input int  BrickSize    = 100;   // Brick Size
input bool ShowWicks    = true;  // Show Wicks
input bool TotalBars    = false; // Use full history
input bool SwapOpenClose = false;// Swap Open & Close
input int Reserve = 0;

int   BarsCount   = 500;         // Bars Count

// indicator buffers
double         OpenBuffer[];
double         HighBuffer[];
double         LowBuffer[];
double         CloseBuffer[];
MqlRates       renkoBuffer[];

double         brickSize, upWick, downWick;

datetime lastBar;
int shift;

#include <NaNs.mqh>

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int OnInit()
{
   // Renko brick size
   brickSize = BrickSize * _Point;
   // Minimum brick size
   if(brickSize < _Point)
   {
      brickSize = _Point;
      Print("Brick size must be a minimum value of ", brickSize);
   }

   // optimal bars count
   if(TotalBars == true && Bars(_Symbol, PERIOD_M1))
      BarsCount = Bars(_Symbol, PERIOD_M1); // total count of bars
   else if(brickSize > 1)
      BarsCount = BarsCount * (int)brickSize;
   
   upWick = downWick = 0;

   //-------------------------------------------------------
   brickSize = NormalizeDouble(brickSize, _Digits);
   // Buffer mapping
   SetIndexBuffer(0, OpenBuffer, INDICATOR_DATA);
   SetIndexBuffer(1, HighBuffer, INDICATOR_DATA);
   SetIndexBuffer(2, LowBuffer, INDICATOR_DATA);
   SetIndexBuffer(3, CloseBuffer, INDICATOR_DATA);
   // Array series
   ArraySetAsSeries(OpenBuffer, true);
   ArraySetAsSeries(HighBuffer, true);
   ArraySetAsSeries(LowBuffer, true);
   ArraySetAsSeries(CloseBuffer, true);
   ArraySetAsSeries(renkoBuffer, true);
   
   PlotIndexSetDouble(0, PLOT_EMPTY_VALUE, EMPTY_VALUE);
   if(SwapOpenClose) PlotIndexSetString(0, PLOT_LABEL, "Close;High;Low;Open");
   PlotIndexSetInteger(0, PLOT_LINE_COLOR, 1, (int)ChartGetInteger(0, CHART_COLOR_BACKGROUND));
   // Levels
   IndicatorSetInteger(INDICATOR_LEVELS, 1);
   IndicatorSetInteger(INDICATOR_LEVELCOLOR, clrGray);
   IndicatorSetInteger(INDICATOR_LEVELSTYLE, STYLE_SOLID);
   IndicatorSetInteger(INDICATOR_DIGITS, _Digits);
   // Title
   string title = "bLuErEnKoBaRs [" + (string)BrickSize + " pts]";
   IndicatorSetString(INDICATOR_SHORTNAME, title);
   
   if(Reserve > 0)
   {
     shift = Reserve;
     PlotIndexSetInteger(0, PLOT_SHIFT, shift);
   }

   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void OnTimer()
{
  EventKillTimer();
  ChartSetSymbolPeriod(0, NULL, 0); // request refresh
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,      // size of input time series
                const int prev_calculated,  // bars handled on a previous call
                const datetime &time[],     // Time
                const double &open[],       // Open
                const double &high[],       // High
                const double &low[],        // Low
                const double &close[],      // Close
                const long &tick_volume[],  // Volume Tick
                const long &volume[],       // Volume Real
                const int &spread[]         // Spread
               )
{
   ArraySetAsSeries(open, true);
   ArraySetAsSeries(high, true);
   ArraySetAsSeries(low, true);
   ArraySetAsSeries(close, true);
   ArraySetAsSeries(time, true);
   
   int size = ArraySize(renkoBuffer);

   // Calculate once
   if(size == 0)
   {
      if(Reserve < 0 && rates_total > TerminalInfoInteger(TERMINAL_MAXBARS))
      {
        PrintFormat("Warning: rates_total (%d) > TERMINAL_MAXBARS (%d)", rates_total, TerminalInfoInteger(TERMINAL_MAXBARS));
      }
   
      MqlRates m1Buffer[];
      ArraySetAsSeries(m1Buffer, true);
      const int total = CopyRates(_Symbol, PERIOD_M1, 0, BarsCount, m1Buffer);

      // Fail
      if(total < 0)
      {
         Comment("Failed to copy history data for the M1 time frame. Error #", _LastError, ". Retry...");
         EventSetTimer(1);
         return(0);
      }

      // Start
      Comment("Reading history... 0% [0 of ", total, "]");

      // Fill Renko history based on M1 OHLC data
      double gap, up, down, progress;
      for(int i = total - 2; i >= 0; i--)
      {
         gap = MathAbs(m1Buffer[i].open - m1Buffer[i + 1].close);
         up = m1Buffer[i].high - MathMax(m1Buffer[i].open, m1Buffer[i].close);
         down = MathMin(m1Buffer[i].open, m1Buffer[i].close) - m1Buffer[i].low;
         // Renko bricks
         if(gap > brickSize) Renko(m1Buffer[i].open);
         // If positive candle, Lo-Hi
         if(m1Buffer[i].open < m1Buffer[i].close)
         {
            if(down > brickSize) Renko(m1Buffer[i].low);
            if(up > brickSize) Renko(m1Buffer[i].high);
         }
         else     // Else Hi-Lo
         {
            if(up > brickSize) Renko(m1Buffer[i].high);
            if(down > brickSize) Renko(m1Buffer[i].low);
         }
         Renko(m1Buffer[i].close);
         // Progress
         progress = 100 - i * 100 / total;
         if(i % 50 == 0) Comment("Reading history... ", progress, "% [", total - i, " of ", total, "]");
      }
      Comment("");
   }
   else
   {
      if(lastBar != time[0]) // new bar added
      {
        if(!Reserve)
        {
          ChartSetSymbolPeriod(0, _Symbol, _Period);
          #ifdef _DEBUG
          Print("Refresh by bar");
          #endif
        }
        else
        {
          if(Reserve > 0)
          {
            PlotIndexSetInteger(0, PLOT_SHIFT, ++shift);
            Comment("++", shift);
          }
          else
          {
            OpenBuffer[1] = EMPTY_VALUE; // clear NaN
          }
          OpenBuffer[0] = EMPTY_VALUE;
          HighBuffer[0] = EMPTY_VALUE;
          LowBuffer[0] = EMPTY_VALUE;
          CloseBuffer[0] = EMPTY_VALUE;
        }
      }
   }

   // Current tick
   const int prev = Renko(close[0]);
   size = ArraySize(renkoBuffer);
   lastBar = time[0];

   //----------------------------------------------------------------------
   // Calculation of the starting number 'first' for the cycle of recalculation of bars

   int first;
   if(prev_calculated == 0) // checking for the first start of the indicator calculation
   {
      ArrayInitialize(OpenBuffer, EMPTY_VALUE);
      ArrayInitialize(HighBuffer, EMPTY_VALUE);
      ArrayInitialize(LowBuffer, EMPTY_VALUE);
      ArrayInitialize(CloseBuffer, EMPTY_VALUE);
      first = (rates_total > size) ? size : rates_total; // starting number for calculation of all bars
   }
   else
   {
      first = size - prev + 2; // size;
   }

   if(Reserve < 0)
   {
     shift = rates_total - size;
     if(shift < 0) Print("Renko buffer overflow, will terminate...");
   }
   
   for(int i = first - 2; i >= 0; i--)
   {
      OpenBuffer[shift + i + 1] = SwapOpenClose ? renkoBuffer[i].close : renkoBuffer[i].open;
      HighBuffer[shift + i + 1] = renkoBuffer[i].high;
      LowBuffer[shift + i + 1] = renkoBuffer[i].low;
      CloseBuffer[shift + i + 1] = SwapOpenClose ? renkoBuffer[i].open : renkoBuffer[i].close;
      
      if(i == 0) // the forming box is not stored in renkoBuffer yet
      {
         OpenBuffer[shift + i] = SwapOpenClose ? close[i] : renkoBuffer[i].close;
         HighBuffer[shift + i] = upWick ? upWick : MathMax(renkoBuffer[i].close, renkoBuffer[i].open);
         LowBuffer[shift + i] = downWick ? downWick : MathMin(renkoBuffer[i].close, renkoBuffer[i].open);
         CloseBuffer[shift + i] = SwapOpenClose ? renkoBuffer[i].close : close[i];
      }
   }

   IndicatorSetDouble(INDICATOR_LEVELVALUE, 0, close[0]);
   if(Reserve < 0)
   {
     OpenBuffer[0] = NaNs[NAN_QUIET + size];
   }
   
   // Print(__FILE__, " ", (Reserve < 0 ? size + 1 : rates_total), " ", prev_calculated, " ", ChartGetInteger(0, CHART_VISIBLE_BARS, 0));
   return rates_total; // (Reserve < 0 ? size + 1 : rates_total);
}
//+------------------------------------------------------------------+
//| Add one slot to array series and shift 1                         |
//+------------------------------------------------------------------+
void RenkoAdd()
{
   int size = ArraySize(renkoBuffer);
   ArraySetAsSeries(renkoBuffer, false);
   ArrayResize(renkoBuffer, size + 1, 10000);
   ArraySetAsSeries(renkoBuffer, true);
   const static MqlRates zero = {0, EMPTY_VALUE, EMPTY_VALUE, EMPTY_VALUE, EMPTY_VALUE};
   renkoBuffer[0] = zero;
   //ArrayCopy(renkoBuffer, renkoBuffer, 1, 0, size);
   if(lastBar)
   {
     if(!Reserve)
     {
       ChartSetSymbolPeriod(0, _Symbol, _Period);
       #ifdef _DEBUG
       Print("Refresh on box");
       #endif
     }
     else if(Reserve > 0)
     {
       PlotIndexSetInteger(0, PLOT_SHIFT, --shift);
       Comment("--", shift);
     }
   }
}
//+------------------------------------------------------------------+
//| Add a up renko bar                                               |
//+------------------------------------------------------------------+
void RenkoUp(double points)
{
   RenkoAdd();
   renkoBuffer[0].open = renkoBuffer[1].close + points - brickSize;
   renkoBuffer[0].high = renkoBuffer[1].close + points;
   renkoBuffer[0].low = renkoBuffer[1].close;
   if(ShowWicks && downWick > 0 && downWick < renkoBuffer[1].close) renkoBuffer[0].low = downWick;
   else renkoBuffer[0].low = renkoBuffer[0].open;
   renkoBuffer[0].close = renkoBuffer[1].close + points;
   upWick = downWick = renkoBuffer[0].close;
}
//+------------------------------------------------------------------+
//| Add a down renko bar                                             |
//+------------------------------------------------------------------+
void RenkoDown(double points)
{
   RenkoAdd();
   renkoBuffer[0].open = renkoBuffer[1].close - points + brickSize;
   if(ShowWicks && upWick > renkoBuffer[1].close) renkoBuffer[0].high = upWick;
   else renkoBuffer[0].high = renkoBuffer[0].open;
   renkoBuffer[0].low = renkoBuffer[1].close - points;
   renkoBuffer[0].close = renkoBuffer[1].close - points;
   upWick = downWick = renkoBuffer[0].close;
}
//+------------------------------------------------------------------+
//| Add renko bars                                                   |
//+------------------------------------------------------------------+
int Renko(double price)
{
   const int size = ArraySize(renkoBuffer);

   upWick = MathMax(upWick, price);
   downWick = MathMin(downWick, price);

   if(size == 0)
   {
      // First brick
      RenkoAdd();
      renkoBuffer[0].high = renkoBuffer[0].close = NormalizeDouble(MathFloor(price / brickSize) * brickSize, _Digits);
      renkoBuffer[0].open = renkoBuffer[0].low = renkoBuffer[0].close - brickSize;
   }
   else if(size < 2)
   {
      // Up
      for(; price > renkoBuffer[0].close + brickSize;)
         RenkoUp(brickSize);
      // Down
      for(; price < renkoBuffer[0].close - brickSize;)
         RenkoDown(brickSize);
   }
   else
   {
      // Up
      if(renkoBuffer[0].close > renkoBuffer[1].close)
      {
         if(price > renkoBuffer[0].close + brickSize)
         {
            for(; price > renkoBuffer[0].close + brickSize;)
               RenkoUp(brickSize);
         }
         // Down 2x
         else if(price < renkoBuffer[0].close - 2 * brickSize)
         {
            RenkoDown(2 * brickSize);
            for(; price < renkoBuffer[0].close - brickSize;) // was [1] - critial bug
               RenkoDown(brickSize);
         }
      }
      // Down
      if(renkoBuffer[0].close < renkoBuffer[1].close)
      {
         if(price < renkoBuffer[0].close - brickSize)
         {
            for(; price < renkoBuffer[0].close - brickSize;)
               RenkoDown(brickSize);
         }
         // Up 2x
         else if(price > renkoBuffer[0].close + 2 * brickSize)
         {
            RenkoUp(2 * brickSize);
            for(; price > renkoBuffer[0].close + brickSize;)
               RenkoUp(brickSize);
         }
      }
   }
   return size;
}
//+------------------------------------------------------------------+
